
#define LD34_WIDTH 320
#define LD34_HEIGHT 200
#define LD34_GAME_OBJ_MAX 8

struct ld34_game
{
	struct m_image img[LD34_GAME_OBJ_MAX];
	struct h_scene vecto[LD34_GAME_OBJ_MAX];
	float2 img_pos[LD34_GAME_OBJ_MAX];
	float2 vecto_pos[LD34_GAME_OBJ_MAX];
	int img_count;
	int vecto_count;

	int scene;
	int step;

	int smell;
	int whisper;
	int thrust;
	int key;

} game;

struct m_image ld34_font = M_IMAGE_IDENTITY();
struct m_image ld34_ui[8];
double ld34_t;
double _t = 0;

float ld34_color_pl[3] = {0.75, 0.75, 1.0};
float ld34_color_ch[3] = {0.0, 0.0, 0.0};
float ld34_color_ch_dark[3] = {0.25, 0.25, 0.25};

void ld34_game_destroy();
void ld34_scale_vecto();
void ld34_init_intro();
void ld34_init_sc02();
void ld34_init_sc03();
void ld34_init_sc04();
void ld34_init_sc05();
void ld34_init_sc06();
void ld34_init_sc07();

#include "ld34_sc02.c"
#include "ld34_sc03.c"
#include "ld34_sc04.c"
#include "ld34_sc05.c"
#include "ld34_sc06.c"
#include "ld34_sc07.c"


void ld34_init()
{
	ld34_t = ctoy_get_time();

	// game
	memset(&game, 0, sizeof(struct ld34_game));
	memset(&ld34_ui, 0, sizeof(ld34_ui));

	// font
	img_load_float(&ld34_font, "dataset/font/small.tga");

	// ui
	img_load_float(&ld34_ui[0], "dataset/ui/key_1.png");
	img_load_float(&ld34_ui[1], "dataset/ui/key_2.png");
}

double ld34_get_elapsed_t()
{
	double t = ctoy_get_time();
	double elapst = t - ld34_t;
	ld34_t = t;
	return elapst;
}

void ld34_game_destroy()
{
	int i;
	for (i = 0; i < game.img_count; i++)
		m_image_destroy(&game.img[i]);
	for (i = 0; i < game.vecto_count; i++)
		h_destroy(&game.vecto[i]);
	game.vecto_count = 0;
	game.img_count = 0;
	game.step = 0;
}

void ld34_destroy()
{
	int i;

	ld34_game_destroy();
	m_image_destroy(&ld34_font);
	for (i = 0; i < 8; i++)
		m_image_destroy(ld34_ui);
}

void ld34_scale_vecto()
{
	float2 scale = {LD34_WIDTH - 1, LD34_HEIGHT - 1};
	int i;

	for (i = 0; i < game.vecto_count; i++)
		h_scale(&game.vecto[i], &scale);
}

void ld34_init_intro()
{
	ld34_game_destroy();

	if(0){
		//game.scene = 1;
		//ld34_init_sc02();

		//game.scene = 2;
		//ld34_init_sc03();

		//game.scene = 5;
		//ld34_init_sc06();

		//game.scene = 4;
		//ld34_init_sc05();

		game.scene = 6;
		ld34_init_sc07();
		return;
	}

	game.img_count = 1;
	img_load_float(&game.img[0], "dataset/intro/bg.png");

	game.vecto_count = 1;
	h_load(&game.vecto[0], "dataset/intro/sc01.hou_raw");

	ld34_scale_vecto();
	_t = 0;
}

void ld34_update_intro(struct m_image *dest, double t)
{
	int ti15 = t * 10;

	m_image_copy(dest, &game.img[0]);

	if (ti15 > 20) {
		int i = ti15 - 20;
		if (i < game.vecto[0].layer[0].frame_count)
		h_frame_draw(dest, game.vecto[0].layer[0].frame + i);
	}

	float v = M_MIN(1, t * 0.2);
	m_image_mul1(dest, dest, v);

	if (ti15 > 40)
		m_font_draw(dest, &ld34_font, "I hate space travel...", ti15 - 40, 12, 182, ld34_color_pl);

	if (t > 7.5) {
		game.scene++;
		ld34_init_sc02();
	}
}

void ld34_update(struct m_image *dest)
{
	double elapst = ld34_get_elapsed_t();

	_t += elapst;

	switch(game.scene) {
	case 0:
		ld34_update_intro(dest, _t);
		break;
	case 1:
		ld34_update_sc02(dest, _t);
		break;
	case 2:
		ld34_update_sc03(dest, _t);
		break;
	case 3:
		ld34_update_sc04(dest, _t);
		break;
	case 4:
		ld34_update_sc05(dest, _t);
		break;
	case 5:
		ld34_update_sc06(dest, _t);
		break;
	case 6:
		ld34_update_sc07(dest, _t);
		break;
	default:
		memset(dest->data, 0, sizeof(float) * dest->size);
		break;
	}
}
